/*
 * coverter.c - Three functions for converting a morse "string" like
 * .- to its ASCII character.
 */

#include "twcw.h"

unsigned char letters[512];  /* has some room to grow */
int val, cnt;


/*
 * next_symbol - given a '.' or '-', build the "trinary like" value 
 * for a morse character.  Use get_letter below to retrive
 * the value.
 */
void next_symbol (char sym)
{
   cnt++;
   if (cnt > 8)    /* too many */
   {
      printf ("next_symbol - Symbol too long - Starting over\n");
      cnt = 0;                       /* do housekeeping */
      val = 0;
      return;
   }

   if (sym == '.' || sym == '-')     /* do the double dabble */
   {
      if (sym == '.')
      {
         val = val + 1;              /* add value of sym */
      }
      else
      {
         val = val + 2;              /* add value of sym */
      }
      val = val << 1;                /* shift by base */
   }
   else
   {
      fprintf (stderr, "decoder - invalid input character\n");
      return;
   }
}


/*
 * get_letter - returns the current value for the morse string
 */
char get_letter()
{
   int rtn;

   rtn = val >> 1;                   /* must undo last shift */ 
   cnt = 0;                          /* do housekeeping */
   val = 0;
   return (letters[rtn]);
}


/*
 * build_table - builds an array of chars.  Table is used by get_letter()
 * to get the ASCII char.
 */
void init_table (void)
{
   int i;

   for (i = 0; i < 512; i++)
      //letters[i] = '=';
      letters[i] = (char) 0;

   letters[1]   = 'E';
   letters[2]   = 'T';
   letters[3]   = 'I';
   letters[4]   = 'A';
   letters[5]   = 'N';
   letters[6]   = 'M';
   letters[7]   = 'S';
   letters[8]   = 'U';
   letters[9]   = 'R';
   letters[10]  = 'W';
   letters[11]  = 'D';
   letters[12]  = 'K';
   letters[13]  = 'G';
   letters[14]  = 'O';
   letters[15]  = 'H';
   letters[16]  = 'V';
   letters[17]  = 'F';
   letters[19]  = 'L';
   letters[21]  = 'P';
   letters[22]  = 'J';
   letters[23]  = 'B';
   letters[24]  = 'X';
   letters[25]  = 'C';
   letters[26]  = 'Y';
   letters[27]  = 'Z';
   letters[28]  = 'Q';
   letters[31]  = '5';
   letters[32]  = '4';
   letters[34]  = '3';
   letters[38]  = '2';
   letters[39]  = '%';
   letters[41]  = '@';
   letters[46]  = '1';
   letters[47]  = '6';
   letters[48]  = '-';
   letters[49]  = '/';
   letters[53]  = '$';
   letters[55]  = '7';
   letters[59]  = '8';
   letters[61]  = '9';
   letters[62]  = '0';
   letters[62]  = '';
   letters[68]  = '#';
   letters[75]  = '?';
   letters[84]  = '.';
   letters[114] = ',';
   letters[196] = '^';
   letters[255] = '!';
}


#if 0
/*
 * Given a dit-dah string, prints the value and letter
 */
int main (int argc, char *argv[])
{
   int i;
   char letter;

   init_table();

   for (i=0; i<strlen(argv[1]); i++)
   {
      if (argv[1][i] == '~')
         break;
      next_symbol (argv[1][i]);
   }
   letter = get_letter();
   printf ("letter=%c\n", letter);
   return 0;
}
#endif
