/*
 * rcvCW.c
 */

#include "twcw.h"
#include "common.h"

#define EMPTY 0
#define FULL 1
#define CHANNELS 1

/* window goodies */
Display *display;
Window window;
int screen;
Colormap cmap; 
GC gc;
XColor redPix, defPix;
rfftw_plan planb;

int buffer_state;
//int red_xpos_lo = (2 * RX_FREQ * SAMPLE_SIZE / SAMPLE_FREQ) - 6;
//int red_xpos_hi = (2 * RX_FREQ * SAMPLE_SIZE / SAMPLE_FREQ) + 6;
int red_xpos_lo = (RX_FREQ / BIN_SIZE) - 6;
int red_xpos_hi = (RX_FREQ / BIN_SIZE) + 6;

void getSamp (unsigned int sample[]);
void drawSA (fftw_real power[]);
void doFFT (int in[], int out[]);
void filter (fftw_real power[]);

extern int sens;

/*
 * init_audio - sets up the soundcard.  Also initializes vars for 
 * drawing.
 */
int init_audio ()
{
   int  val;
   int full_duplex = 0;

   /* First, the drawing stuff */
   display = XtDisplay (da1);
   window = XtWindow (da1);
   screen = DefaultScreen (display);
   cmap = DefaultColormap (display, screen); 
   gc = XDefaultGC (display, screen);
   XAllocNamedColor (display, cmap, "red", &redPix, &defPix);
   planb = rfftw_create_plan(SAMPLE_SIZE, FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE);

   /* Open Sound Card */
   if ((audio_fd = open (AUDIO_PATH, O_RDWR | O_NONBLOCK)) < 0)
   {
      if (errno == EBUSY)
      {
         printf ("\n\nSoundcard is busy! - try again later.\n");
         exit (1);
      }
      else
      {
         perror (AUDIO_PATH);
         exit (1);
      }
   }

   /* Set Audio Format */
   val = AFMT_U8;
   if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &val) < 0)
   {
      perror ("Set AFMT error");
   }

   if (val == AFMT_U8)
      printf ("AFMT_%d\n", val);
   else
   {
      printf ("Error setting AFMT\n");
      exit (1);
   }

   /* Set Mono/Stereo Mode */
   val = CHANNELS;
   if (ioctl(audio_fd, SNDCTL_DSP_STEREO, &val) < 0 )
   {
      perror ("Set MONO/STEREO error\n");
      exit (1);
   }

   if (val != CHANNELS)
   {
      printf ("Can\'t set MONO/STEREO\n");
      exit (1);
   }


   /* Set Sample Freq */
   val=SAMPLE_FREQ;
   if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &val) < 0)
   {
      perror ("Set Sample Freq");
      exit (1);
   }

   if(val != SAMPLE_FREQ)
   {
      fprintf(stderr,"inexact sampling rate: "
                     "request for %d resulted in %d",SAMPLE_FREQ,val);
   }

   val = 1;
   if (ioctl(audio_fd, SNDCTL_DSP_NONBLOCK, &val) < 0 )
      return -6;

   // Check if the device is operating in full duplex mode
   if (ioctl(audio_fd, SNDCTL_DSP_GETCAPS, &val) < 0)
      perror("Warning: GETCAPS on audio device failed");
   else
      if(val&DSP_CAP_DUPLEX)
         full_duplex=1;

   #if 1
   fprintf(stderr,"Using %s mode!\n",
           full_duplex?"full duplex":"halv duplex");
   #endif

   return 0;
}


void init_mixer ()
{
   int mixer_fd, val;

   mixer_fd = open(MIXER_PATH, O_RDWR);

   if (mixer_fd < 0)
   {
      perror ("twcw - Can't open mixer");
      exit (1);
   }

   val=SOUND_MASK_LINE;
   //val=SOUND_MASK_MIC;
   if (ioctl (mixer_fd, SOUND_MIXER_WRITE_RECSRC, &val) < 0)
   {
      perror ("mixer recsrc");
      fprintf (stderr, "mixer recsrc failed\n");
   }

//TJW
#if 0
   val = 0;                /* nothing out */
   //val = SOUND_MASK_PCM;   /* PCM out */
   //val = SOUND_MASK_LINE;  /* line in to output source */
   //val = SOUND_MASK_MIC;   /* mic in to output source */
   if (ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_OUTSRC), &val) < 0)
   {
      perror ("mx out");
      fprintf (stderr, "mixer outsrc failed\n");
   }
#endif
   close (mixer_fd);
}


/*
 * close_sound
 */
void close_sound (void)
{
   if (close (audio_fd) == -1)
      printf ("Close error\n");
}


/*
 * select_loop
 */
void select_loop (void)
{
   int res;
   fd_set rset, wset, eset;
   struct timeval tm;
   static unsigned int sample[SAMPLE_SIZE];   //TJW a fftw_real?
   static fftw_real power[SAMPLE_SIZE/2+1];

   FD_ZERO(&rset); 
   FD_ZERO(&wset);
   FD_ZERO(&eset);
   FD_SET(audio_fd, &rset);

   tm.tv_sec = 0;      /* 0 sec */
   tm.tv_usec = 0;     /* 0 ms */

   res=select(audio_fd+1, &rset, &wset, &eset, &tm);

   switch (res)
   {
      case -1:
         perror ("select");
         exit;

      case 0:
         /* do something constructive? */
         usleep (1);
         return;

      case 1:
         getSamp (sample);
         if (buffer_state == FULL)
         {
            doRFFTW (sample, power);
            #if 1              // xform it back for filtered?  TJW
              filter (power);
            #endif
            drawSA (power);
            decode (power, red_xpos_lo, red_xpos_hi);
            buffer_state = EMPTY;
         }
         return;

      default:
         printf ("Busted!\n");
         exit (666);
   }
}


/*
 * getSamp - reads SAMPLE_SIZE chars of audio
 */
void getSamp (unsigned int sample[])
{
   int rtn;

   if (buffer_state == FULL)
   {
      printf ("overran buffer\n");
      buffer_state = EMPTY;
      return;
      // -or- a fatal exit(1);  ???
   }

   rtn = read (audio_fd, sample, SAMPLE_SIZE);
   if (rtn == SAMPLE_SIZE)
   {
      buffer_state = FULL;
      return;
   }
   else
   {
      if (errno == EAGAIN)
      {
         perror ("EAGAIN");
         return;
      } 

      if(errno == EINTR)
      {
         perror ("EINTR");
         exit(1);
      }
      exit;
   }
}


/*
 * draw SA
 */
void drawSA (fftw_real power[])
{
   int x;

   XClearArea (display, window,
               0, 0,
               red_xpos_lo - 1, DA_HEIGHT - sens - 1,
               0);

   XClearArea (display, window,
               red_xpos_hi + 1, 0,
               SAMPLE_SIZE / 2, DA_HEIGHT - sens - 1,
               0);

   XClearArea (display, window,
               0, DA_HEIGHT - sens + 1,
               red_xpos_lo - 1, sens,
               0);

   XClearArea (display, window,
               red_xpos_hi + 1, DA_HEIGHT - sens + 1,
               SAMPLE_SIZE / 2, sens,
               0);

   XClearArea (display, window,
               red_xpos_lo + 1, 0,
               red_xpos_hi - red_xpos_lo - 2, DA_HEIGHT,
               0);

   XSetForeground (display, gc, WhitePixel(display, screen));
   for (x=0; x<SAMPLE_SIZE/2; x++)
   {
      XDrawLine (display, window, gc, x, DA_HEIGHT, x, DA_HEIGHT - power[x]);
   }

   XSetForeground (display, gc, redPix.pixel);
   XSetLineAttributes (display, gc, 2, LineSolid, CapRound, JoinRound);

   /* draw tune lines */
   XDrawLine (display, window, gc, red_xpos_lo, 0, red_xpos_lo, DA_HEIGHT);
   XDrawLine (display, window, gc, red_xpos_hi, 0, red_xpos_hi, DA_HEIGHT);

   /* draw sens line - really two lines */
   XDrawLine (display, window, gc,
              0, DA_HEIGHT - sens,
              red_xpos_lo, DA_HEIGHT - sens);

   XDrawLine (display, window, gc,
              red_xpos_hi, DA_HEIGHT - sens,
              DA_WIDTH, DA_HEIGHT - sens);

   XSetForeground (display, gc, WhitePixel(display, screen));
   XSetLineAttributes (display, gc, 1, LineSolid, CapRound, JoinRound);
}

/*
 * filter
 */
void filter (fftw_real input[])
{
#if 1
   int i;
   fftw_real outf[SAMPLE_SIZE/2];
   unsigned int out[SAMPLE_SIZE/2]; 
      
   //rfftw_one(planb, input, outf);

   for (i=0; i<SAMPLE_SIZE/2 + 1; i++)
   {
      //out[i] = outf[i];
      //printf ("%d ", out[i]);
   }

   //write (audio_fd, out, SAMPLE_SIZE);
#endif
}
