/*
 * twcw:  A gui application that sends Morse Code 
 * Copyright (C) 1997 Ted Williams WA0EIR 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Versions: 1.2 - APR 2004
 */

/*
 * main FUNCTION 
 * child process that sends CW via RTS line on the serial port.
 */
#include "common.h"
#include "sendCW.h"

int cwirc_shm_id;
struct cwirc_extension_api *cwirc_api_shm;

int  main (int argc, char *argv[])
{
   int rtn;
   struct tag1 message;
   int type = 0;
   int keymode;
   char ser_dev[20] = "\0";

   /* did you try to execute sendCW standalone? */
   if (argc != 3)
   {
     fprintf (stderr, "%s can not run standalone.\n", argv[0]);
     fprintf (stderr, "It must be started by running twcw.\n");
     exit (1);
   }

   keymode = atoi (argv[1]);
   fwpm = FARNS_OFF;
   /*
    * Check to see if we are using TWCW or CWIRC keymode 
    */
   switch (keymode)
   {
      case TWCW:
         fprintf (stderr, "sendCW: using TWCW keymode\n");
         key_mode = TWCW;
         strcpy (ser_dev, argv[2]);                   /* keydev to ser_dev */
         break;

      case CWIRC:
         fprintf (stderr, "sendCW: using cwirc keymode\n");
         key_mode = CWIRC;
         cwirc_shm_id = strtol (argv[2] ,NULL, 0);    /* keydev to shm_id */
         cwirc_api_shm = shmat(cwirc_shm_id,NULL,0);

         if (cwirc_api_shm == (struct cwirc_extension_api *) -1)
            fprintf(stderr, "sendCW: Can't attach the CWirc shared memory\n");
         break;

      default:
         fprintf (stderr, "sendCW - Invalid keymode - must CWIRC or TWCW\n");
         exit (1);
   }

   /*
    * trap the following signals for the cleanup function
    */
   signal (SIGINT,  cleanup);
   signal (SIGTERM, cleanup);
   signal (SIGQUIT, cleanup);
   signal (SIGSEGV, cleanup);

   /*
    * Initialize letters array and connect to the message queue
    */
   initialize (ser_dev);

   if ((qid = msgget (TWCW_KEY, 0666)) < 0)
   {
      perror ("msgget");
      exit (1);
   }

   /*
    * Read a Message From the Queue
    */
   while (1)
   {
      rtn = msgrcv (qid, (struct msgbuf *) &message, MAX_WORD_LENGTH, type, 0);
      if (rtn < 0 && errno == EAGAIN)
      {
         perror ("sendCW msgrcb");
      }

      switch (message.type)
      {
         case WORD_MSG:                      /* Got a word to send */
            doWord (message.data.word);
            continue;

         case WPM_MSG:                       /* Sets a new WPM */
            setWPM (message.data.val);
            continue;

         case TONE_MSG:
            setTone (message.data.val);      /* Sets the tone freq */
            continue;

         case SIDETONE_MSG:
            setSidetone (message.data.val);  /* Turns sidetone on and off */
            continue;

         case FARNS_MSG:                     /* set Farnsworth value */
            fwpm = message.data.val;
            setFarns (fwpm);                  /* calc any new ic and il times */
            continue;

         case STOP_MSG:
            if (message.data.word[0] == '0')
            {
               type = -STOP_MSG;             /* Stop sending */ 
            }
            else
            {
               type = -WORD_MSG;             /* Start sending */
            }
      }
   }
   exit (0);
}
