/*
 * twcw:  A gui application that sends Morse Code 
 * Copyright (C) 1997 Ted Williams WA0EIR 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Version 1.2  APR 2004
 */

/*
 * main Function
 *  main calls the following functions:
 *   1) build_widgets() to create the interface.
 *   2) setup() to initialize a message queue and signal traping.
 *   3) fork_sendCW() to create the child process.
 *  main then adds a work proceedure (do_work) and enters the event loop.
 */
#include "common.h"
#include "twcw.h"
#include "vibroplex.xpm"

/* Global */
AppRes app_res;

int  main (int argc, char *argv[])
{
   int i;
   Widget iconwin;
   Pixmap pixmap, mask;
   XpmAttributes pix_attributes;
   XpmColorSymbol transparentColor[1] = {{NULL, "none", 0}};
   struct stat buf;
   Pixel bg;
   Dimension width, height;
   struct tag1 dummy;

/*
 * Describe the application defined resources. This struct is
 * for the app resources which are read from the resource
 * file.  See the resource file (Twcw) for more info.
 */
   XtResource app_res_desc[] =
      {
         {
         XmNcall,                             /* Name */
         XmCCall,                             /* Class */
         XmRString,                           /* Target data type */
         sizeof (char *),                     /* size of target type */
         XtOffsetOf (AppRes, call),           /* Offset into LogRes */ 
         XtRImmediate, "N0CALL"               /* Default data type */
         },
 
         {
         XmNbuttonNames,                      /* Name */
         XmCButtonNames,                      /* Class */
         XmRStringTable,                      /* Target data type */
         sizeof (char **),                    /* size of target type */
         XtOffsetOf (AppRes, buttonNames),    /* Offset into LogRes */ 
         XtRImmediate, "Bogus Names"          /* Default value */
         },	 

         {
         XmNserialDevice,                     /* Name */
         XmCSerialDevice,                     /* Class */
         XmRString,                           /* Target data type */
         sizeof (char *),                     /* size of target type */
         XtOffsetOf (AppRes, serialDevice),   /* Offset into LogRes */ 
         XtRImmediate, ""                     /* Default value */
         }	
      };

   /* seed the random number generator for practice mode */
   srand (time(NULL));

   /*
    * Initialize the tool kit, create the application and icon shells,
    * get the app defined resources and check that dirPath and 
    * buttonNames are defined.  This does not check for valid
    * pathnames.  They can be NULL - just gota have twcw.dirPath
    * and twcw.buttonNames in Twcw.
    */   
   shell = XtVaAppInitialize (&ac, "Twcw", NULL, 0, &argc, argv, NULL,
      XmNmwmFunctions, MWM_FUNC_ALL | MWM_FUNC_CLOSE,
      XmNiconName, "TWCW",
      NULL);

   /*
    * Create Icon window and its pixmap
    */
   iconwin = XtVaAppCreateShell ("Iconwin", "shell",
                wmShellWidgetClass, XtDisplay(shell),
                XmNmappedWhenManaged, False,
                XmNwidth,  60,
                XmNheight, 40,
                NULL);

   XtVaGetValues (iconwin,
      XmNbackground, &bg,
      NULL);

   transparentColor[0].pixel = bg;
   pix_attributes.closeness = 40000;
   pix_attributes.valuemask = XpmColorSymbols | XpmCloseness;
   pix_attributes.colorsymbols = transparentColor;
   pix_attributes.numsymbols = 1;

   XpmCreatePixmapFromData (XtDisplay(shell),
      DefaultRootWindow (XtDisplay(shell)),
      vibroplex, &pixmap, &mask, &pix_attributes);

   /*
    * Get resources and check to see if buttonNames was set
    */
   XtGetApplicationResources (shell, &app_res, app_res_desc,
      XtNumber (app_res_desc), NULL, 0);

   fprintf (stderr, "Application Resources\n");
   fprintf (stderr, "  Call %s\n", app_res.call);

   i = 0;
   fprintf (stderr, "  File Names: ");
   while (app_res.buttonNames[i] != NULL)
   {
      fprintf (stderr, "%s%s ", app_res.buttonNames[i],
         app_res.buttonNames[i+1] == NULL ? " " : ",");
      i++;
   }
   fprintf (stderr, "\n");

   fprintf (stderr, "  Serial Device %s\n", app_res.serialDevice);

   if (!strcmp ((char *)app_res.buttonNames, "Bogus Names"))
   {
      fprintf (stderr, "twcw: buttonNames resource not found\n");
      exit (1);
   }

   /*
    * Check command line args
    * Look for the "--cwirc" switch so we know what args are needed. 
    */ 
   if ((argc == 3) && (!strcmp (argv[1], "--cwirc")))   /* for CWIRC mode */
   {
      cwArgs.mode = CWIRC;
      strcpy (cwArgs.keydev, argv[2]);
   }
   else                                                 /* for TWCW mode */
   {
      cwArgs.mode = TWCW;
      strcpy (cwArgs.keydev, app_res.serialDevice);
   }

   /*
    * build the widgets
    */
   build_widgets (shell, &app_res, cwArgs.mode);

   /*
    * Initialize dirpath and helppath
    * and check that they are valid
    */
   strcpy (dirpath, getenv ("HOME"));
   strcat (dirpath, TWCWDIR);             /* twcwDir is always in $HOME */

   if (stat (dirpath, &buf) < 0)
   {
      perror ("twcw");
      fprintf (stderr, "twcw: Can't find %s\n", dirpath);
      exit (1);
   }

   strcpy (helppath, "/usr/local/share/twcw");  /* helpfile is always here */
   strcat (helppath, HELPFILE);
   if (stat (helppath, &buf) < 0)
   {
      perror ("twcw");
      fprintf (stderr, "twcw: Can't find %s\n", helppath);
      exit (1);
   }

   /*
    * Setup the message queue. Also sets up signal traps 
    */
   if (setup() == False)
   {
      fprintf (stderr, "twcw: Can't Open Message Queue\n");
      exit (1);
   }

   if ((pid = fork_sendCW (&cwArgs)) == False)
   {
      fprintf (stderr, "twcw: Can't fork send_cw\n");
      exit (1);
   }

   /*
    *  Set min width of the shell and clear the Farnsworth label
    */
   XtVaGetValues (shell,
      XmNwidth, &width,
      XmNheight, &height,
      NULL);

   XtVaSetValues (shell,
      XmNminWidth, width,
      XmNminHeight, height,
      NULL);


   /* force focus to txText */
   XmProcessTraversal (txText, XmTRAVERSE_CURRENT);

   /*
    * Realize and Set pixmap properties in clock_shell and iconwin
    */
   XtRealizeWidget (iconwin);

   XtVaSetValues (iconwin,
      XmNbackgroundPixmap, pixmap,
      NULL);

   XtVaSetValues (shell,
      XmNiconWindow, XtWindow(iconwin),
      NULL);

   /*
    * Initialize fftw and the soundcard
    */
   // TJW 2.0 init_plan  ();   /* should go hr but in audio - it leaks badly! */
   // TJW 2.0 init_table ();
   // TJW 2.0 init_audio ();
   // TJW 2.0 init_mixer ();

   /* wait for message that speed is set. */
   while (True)
   {
      //if (msgrcv (qid, (struct msgbuf *) &dummy,
      if (msgrcv (qid, (struct tag1 *) &dummy,
         MAX_WORD_LENGTH, WPM_DONE, IPC_NOWAIT) != 1)
      {
         XUndefineCursor (XtDisplay (shell), XtWindow (shell));
         break;
      }
   }

   /*
    * add the work proc and enter the main loop
    */
   XtAppAddWorkProc (ac, do_work, (XtPointer) NULL);

   XtAppMainLoop (ac);
   exit (0);
}


/*
 * fork_sendCW Function
 * Forks the send_CW child process and returns it's pid
 */
int  fork_sendCW ( struct tag2 *cwArgs)
{
   int  pid;
   char mode_str[4];

   switch (pid = fork ())
      {
      case -1:                          /* Can't fork a child */
         perror ("fork");
         exit (1);

      case 0:                           /* Child comes here */
         sprintf (mode_str, "%d", cwArgs->mode);
         execlp ("sendCW", "sendCW", mode_str, cwArgs->keydev, NULL);
         perror ("execl");
         break;

      default:                          /* Parent comes here */
         break;
      }
   return (pid);
}


/*
 * do_work Function
 * Work Proceedure to get messages from the sendCW process
 */
Boolean do_work (XtPointer cdata)
{
   int  rtn;
   char group[8];
   struct tag1 indata;
// 2.0 stuff   Boolean txFlag = XmToggleButtonGetState (tb1);

   /*
    * Here we first check for messages, then on Xmit button state
    * Then can get rid of wait loop nr setup - above... 
    * First do receive, then look for a messages.
    */

#if 0 /* TJW more 2.0 stuff */
   if (txFlag == False)
   {
      select_loop ();  //2.0 select loop
      return False;
   }
#endif
   rtn = msgrcv (qid, (struct msgbuf *) &indata,
           MAX_WORD_LENGTH, WPM_DONE, IPC_NOWAIT);

   if (rtn != -1)    /* restore the cursor if we got a WPM_DONE */
   {
      XUndefineCursor (XtDisplay (shell), XtWindow (shell));
   }

   /*
    * Now look for a SENT_MSG
    */
   rtn = msgrcv (qid, (struct msgbuf *) &indata,
           MAX_WORD_LENGTH, SENT_MSG, IPC_NOWAIT);

   if (rtn == -1 && errno != ENOMSG)   /* There was an error, ENOMSG is OK  */
   {
      perror ("twcw: msgrcv");
      exit (1);
   }

   if (rtn > 0 )     /* there was a SENT_MSG */
   {
      /* TJW - add highlight for xmit text hr? */
      /* Add sent text to rxText if TWCW mode */
      if (cwArgs.mode == TWCW)
      {
         XmTextInsert (rxText, XmTextGetLastPosition (rxText),
            indata.data.word);
      }
      pcnt++; 
   }

   /* Finally, see if we are in practice mode */
   /* and should we send another word */
   if (XmToggleButtonGetState(practiceTB) == True)
   {
      if (pcnt == 6)
      {
         send_msg (WORD_MSG, getFive(group));  /* Send the word to the queue */
         pcnt = 0;
      }
   }
   usleep (100);
   return False;                      /* Always register again */      
}  


/*
 * getFive - generate 5 random letters
 */
char *getFive (char grp[])
{
   int k;

   for (k=0; k<5; k++)
   {
      grp[k] = (rand() % 26) + 65;
   }
   grp[5] = ' ';
   grp[6] = '\0';
   return (grp);
}
