/*
 *      timer.c  --  Reference clock for accurate cycle timing
 *      Copyright (C) 1996  Ted Williams - wa0eir
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      Versions: 1.2 -  APR 2004
 */

#include "common.h"
#include "sendCW.h"

#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/rtc.h>

#define RTC_RATE 1024

/*
 * timer FUNCTION
 */
void timer (long usec)
{
   unsigned long rtc_data;
   int fd;
   int i;
   int cnt;

   cnt = usec * RTC_RATE / 1000000.0;
//fprintf (stderr, "%d\n", cnt);
   /* open rtc */
   if ((fd = open ("/dev/rtc", O_RDONLY)) < 0)
   {
      perror ("timer: open /dev/rtc failed");
      exit (errno);
   }

   /* configure rtc for periodic interupts */
   if (ioctl (fd, RTC_IRQP_SET, RTC_RATE) < 0)
   {
      perror ("timer: RTC_IRQP_SET");
      exit (errno);
   }

   if (ioctl (fd, RTC_PIE_ON, 0) < 0)
   {
      perror ("timer: PIE_ON");
      exit (errno);
   }

   i = 0;
   while (cnt - i > 0)
   {
      if (read (fd, &rtc_data, sizeof(rtc_data)) < 0)
      {
         perror ("timer: RTC read");
         exit (errno);
      }
      i = i + (rtc_data >> 8);
//if (rtc_data >> 8 > 1) fprintf (stderr, "%ld ", rtc_data >> 8);
   }
//fprintf (stderr, "\n");
//if (rtc_data >> 8 > 1) fprintf (stderr, "%ld\n", rtc_data >> 8);


#if 0
   if (ioctl (fd, RTC_PIE_OFF, 0) < 0)
   {
      perror ("timer: PIE_ON");
      exit (errno);
   }
#endif
   close (fd);
}
